/*
 * ===========================================================================
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2004 All Rights Reserved.
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 *
 */

/*
 * The "any.any" package denotes what operating system and version
 * the collector runs on. In this sample, it denotes that this collector
 * is able to run on any operating system and version.
 */
package any.any;

import java.util.*;
/*
 * The "com.ibm.jac" package contains the CollectorV2 and Message classes.
 */
import com.ibm.jac.*;
import java.sql.Types;

/*
 * This collector collects attributes of the Java Runtime Environment used by
 * the client.
 *
 * Runs on any platform supported by the Tivoli Security Compliance Manager
 * client.
 */
public class UserJreInfo extends com.ibm.jac.CollectorV2 {
    
    /*
     * Define the release number of the collector.
     *
     * This number is used to track changes to the collector and is
     * increased each time the collector is changed.
     */
    private static final int RELEASE = 1;
    
    /*
     * Define the name of the database table for the collector.
     *
     * Name the table with a name closely related to the collector's class
     * name for easy identification. The table name must be 30 characters or
     * less in length and contain all capital letters.
     *
     * By convention, the start of the table name indicates the operating
     * system or application. End the table name with its version number,
     * such as:  _V1, _V2, _V14
     *
     * Collector tables are created in the database when the collector
     * is registered on the server. Collector tables reside in the
     * JAC_DATA schema. Use JAC_DATA.your_table_name to reference the
     * table in compliance queries.
     *
     * Table name ruler:                         0123ABCDEFGHIJKLMNOPQRSTUVWXYZ
     */
    private static final String[] TABLENAME = { "ANY_USER_JRE_INFO_V1" };
    
    /* Define a description for the collector.
     *
     * The description is displayed when the collector is installed and
     * when the collector is displayed in the administration console.
     * By convention, the description starts with the word "Description:"
     * and consists of a short sentence describing the function of the
     * collector.
     */
    private static final String DESCRIPTION =
	"Description: Returns Java Runtime Environment Information";
    
    /*
     * Define a list of compatible operating systems for the collector.
     *
     * The collector is permitted to run only on the systems specified.
     * "Windows" refers to all Microsoft Windows versions.
     */
    private static final String[] COMPATIBLE_OS =
    {
	"AIX",
	"HP-UX",
	"LINUX",
	"SUNOS",
	"Windows"
    };
    
    /*
     * Define the columns for the collector's database table.
     *
     * For each column provide the following information:
     *
     *   name        : The name of the column. Column names
     *                 are 30 or less alphanumeric characters
     *                 in length.
     *   type        : The type of data in the column. See
     *                 java.sql.Types for valid values.
     *   size        : The size of the data in the column. Not
     *                 required for some data types, in which
     *                 case, the value specified is ignored.
     *   constraints : DB2 SQL constraints on the column.
     *                 Only "not null" is supported.
     */
    private static final CollectorTable.Column[][] TABLE_DEFINITION = {
        {
	    /*
	     * Column name ruler:      0123ABCDEFGHIJKLMNOPQRSTUVWXYZ
	     */
	    new CollectorTable.Column("VENDOR", Types.VARCHAR, 128, "not null"),
	    new CollectorTable.Column("VERSION", Types.VARCHAR, 32, "not null"),
        }
    };
    
    /*
     * Required default constructor for this collector
     */
    public UserJreInfo() {
    }
    
    /*
     * Required method: getCompatibleOS().
     *
     * Returns a String array containing the list of the operating systems
     * that the collector can run on.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any collector.
     */
    public String[] getCompatibleOS() {
        return COMPATIBLE_OS;
    }
    
    /*
     * Required method: getReleaseNumber().
     *
     * Returns the release number of the collector.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any collector.
     */
    public int getReleaseNumber() {
        return RELEASE;
    }
    
    /*
     * Required method: getDescription().
     *
     * Returns a String containing the description of the collector.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any user defined collector.
     */
    public String getDescription() {
        return DESCRIPTION;
    }
    
    /*
     * Required method: getParameters().
     *
     * Returns a Vector of Strings containing the names of the parameters
     * supported by the collector. Return an empty Vector if no parameters
     * are supported.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any collector.
     */
    public Vector getParameters() {
        /*
         * In the sample, an empty Vector is returned because this collector
         * has no parameters associated with it.
         */
        Vector params = new Vector();
        return params;
    }
    
    /*
     * Required method: getTables().
     *
     * Returns an array of class CollectorTable that describes the
     * structure of the collector database tables.  Each database
     * table defined by the collector is represented by an array
     * element.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any collector.
     */
    public CollectorTable[] getTables() {
        /* Create the table using the table data defined previously. */
        CollectorTable[] tables = new CollectorTable[TABLENAME.length];
        for (int i = 0; i < TABLENAME.length; i++) {
            tables[i] = new CollectorTable(TABLENAME[i]);
	    
            for (int j = 0; j < TABLE_DEFINITION[i].length; j++) {
                tables[i].addColumn(TABLE_DEFINITION[i][j]);
            }
        }
	
        return tables;
    }
    
    /*
     * Required method: executeV2().
     *
     * Gathers compliance data from the client.
     *
     * Compliance data is returned as an array of class Message. Each
     * array element represents the data for a database table.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any collector.
     */
    public Message[] executeV2() {
        Message[] messages;
        Vector[] columns;
        CollectorTable[] tables;
        Object[] record;
        String[] headers;
	
        /*
         * Get the table information for this collector.
         */
        tables = getTables();
	
        /*
         * Allocate an array of Vectors to contain the database
         * table column information for this collector. Allocate
         * a vector for each database table used by the collector.
	 */
        columns = new Vector[TABLENAME.length];
	
        /*
         * Allocate the array of Message objects to be returned from the
         * collector. Allocate a Message object for each database table.
         */
        messages = new Message[TABLENAME.length];
	
	/*
	 *  Instantiate the messages.
	 */
        for (int i = 0; i < TABLENAME.length; i++) {
            messages[i] = new Message(TABLENAME[i]);
	    
            columns[i] = tables[i].getColumns();
	    
            /*
             * Fill in the column headers for the current table.
             */
            headers = new String[columns[i].size()];
            for (int j = 0; j < columns[i].size(); j++) {
                headers[j] =
		    ((CollectorTable.Column) columns[i].elementAt(j)).getName();
            }
            messages[i].getDataVector().addElement(headers);
        }
	
        /*
         * Fill in the data records for each table.
         */
        try {
            /*
             * for the first table
             * record is the record for the first table
             */
            record = new Object[columns[0].size()];
            record[0] = System.getProperty("java.vendor");
            record[1] = System.getProperty("java.version");
            messages[0].getDataVector().addElement(record);
	    
        }
        catch (Exception e) {
            /*
             * Return an error message indicating collector failure
             * if an exception is caught. The error message is
             * logged in the client message log.
             */
            return new Message[] {
		errorMessage("Collector error: exceptiom message = "
			     + e.getMessage())
            };
        }
	
        /*
         * Return the gathered collector data.
         */
        return messages;
    }
    
}
