/*
 * ===========================================================================
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2004 All Rights Reserved.
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 *
 */

/*
 * The "unix.multi" package denotes what operating system and version
 * the collector runs on. In this example, this denotes that this collector
 * is able to run on multiple "unix" operating systems, namely: SunOS and HP-UX.
 */
package unix.multi;

/*
 * The "com.ibm.jac" package contains the CollectorV2 and Message classes.
 */
import com.ibm.jac.*;
import java.util.*;
import java.io.*;
import java.sql.Types;

/*
 * This collector reports the configuration parameters of TCP/IP drivers.
 * Uses command /usr/sbin/ndd <DRIVER_NAME> <DRIVER_ATTRIBUTE> to collect
 * configuration setting for a driver.
 */

public class UserNdd extends com.ibm.jac.CollectorV2 {
    /*
     * Define the release number of the collector.
     *
     * This number is used to track changes to the collector and is
     * increased each time the collector is changed.
     */
    private static final int RELEASE = 1;
    
    /*
     * Define the name of the database table for the collector.
     *
     * Name the table with a name closely related to the collector's class
     * name for easy identification. The table name must be 30 characters or
     * less in length and contain all capital letters.
     *
     * By convention, the start of the table name indicates the operating
     * system or application. End the table name with its version number,
     * such as:  _V1, _V2, _V14
     *
     * Collector tables are created in the database when the collector
     * is registered on the server. Collector tables reside in the
     * JAC_DATA schema. Use JAC_DATA.your_table_name to reference the
     * table in compliance queries.
     */
    private static final String[] TABLENAME =
    {
	"UNIX_NDD_V1"
    };
    
    /* Define a description for the collector.
     *
     * The description is displayed when the collector is installed and
     * when the collector is displayed in the administration console.
     * By convention, the description starts with the word "Description:"
     * and consists of a short sentence describing the function of the
     * collector.
     */
    private static final String DESCRIPTION =
        "Description: Reports the configuration parameters of TCP/IP " +
        "drivers.\n Command: /usr/sbin/ndd DRIVER_NAME DRIVER_ATTRIBUTE " +
	"to collect configuration setting for a drive.\n";
    
    /*
     * Define the parameters that the collector supports.
     */
    private static final String[] PARAMETERS =
    {
	"DRIVER_NAME",
	"DRIVER_ATTRIBUTE"
    };
    
    /*
     * Define a list of compatible operating systems for the collector.
     *
     * The collector runs only on Sun Solaris Operating Environment
     * and HP-UX systems.
     */
    private static final String[] COMPATIBLE_OS =
    {
	"SunOS",
	"HP-UX"
    };
    
    /*
     * Define the columns for the collector's database table.
     *
     * For each column provide the following information:
     *
     *   name        : The name of the column. Column names
     *                 are 30 or less alphanumeric characters
     *                 in length.
     *   type        : The type of data in the column. See
     *                 java.sql.Types for valid values.
     *   size        : The size of the data in the column. Not
     *                 required for some data types, in which
     *                 case, the value specified is ignored.
     *   constraints : DB2 SQL constraints on the column.
     *                 Only "not null" is supported.
     */
    private static final CollectorTable.Column[][] TABLE_DEFINITION =
    {
	{
	    /*
	     * Table: UNIX_NDD_V1
	     * Column name ruler:      0123ABCDEFGHIJKLMNOPQRSTUVWXYZ
	     */
	    new CollectorTable.Column("DRIVER", Types.VARCHAR, 60),
	    new CollectorTable.Column("ATTRIBUTE", Types.VARCHAR, 128),
	    new CollectorTable.Column("INTEGER_VALUE", Types.INTEGER, 0)
	}
    };
    
    /*
     * Define the command to be executed on SunOS
     */
    private static final String NDDCOMMAND_SUNOS = "/usr/sbin/ndd";
    
    /*
     * Define the command to be executed on HP-UX
     */
    private static final String NDDCOMMAND_HPUX = "/usr/bin/ndd";
    
    /*
     * Required default constructor for this collector.
     */
    public UserNdd() {
    }
    
    /*
     * Required method: getCompatibleOS().
     *
     * Returns a String array containing the list of the operating systems
     * that the collector can run on.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any collector.
     */
    public String[] getCompatibleOS() {
        return COMPATIBLE_OS;
    }
    
    /*
     * Required method: getReleaseNumber().
     *
     * Returns the release number of the collector.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any collector.
     */
    public int getReleaseNumber() {
        return RELEASE;
    }
    
    /*
     * Required method: getDescription().
     *
     * Returns a String containing the description of the collector.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any user defined collector.
     */
    public String getDescription() {
        return DESCRIPTION;
    }

    /*
     * Required method: getParameters().
     *
     * Returns a Vector of Strings containing the names of the parameters
     * supported by the collector. Return an empty Vector if no parameters
     * are supported.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any collector.
     */
    public Vector getParameters() {
        Vector params = new Vector();
        params.addAll(Arrays.asList(PARAMETERS));
        return params;
    }
    
    /*
     * Required method: getTables().
     *
     * Returns an array of class CollectorTable that describes the
     * structure of the collector database tables.  Each database
     * table defined by the collector is represented by an array
     * element.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any collector.
     */
    public CollectorTable[] getTables() {
        /*
	  Create the table using the table data defined previously.
	*/
        CollectorTable[] tables = new CollectorTable[TABLENAME.length];
        for (int i = 0; i < TABLENAME.length; i++) {
            tables[i] = new CollectorTable(TABLENAME[i]);
        }
	
        for (int i = 0; i < TABLENAME.length; i++) {
            for (int j = 0; j < TABLE_DEFINITION[i].length; j++) {
                tables[i].addColumn(TABLE_DEFINITION[i][j]);
            }
        }
	
        return tables;
    }
    
    /*
     * Required method: executeV2().
     *
     * Gathers compliance data from the client.
     *
     * Compliance data is returned as an array of class Message. Each
     * array element represents the data for a database table.
     *
     * This method is defined as abstract in the CollectorV2 class and must
     * be implemented by any collector.
     */
    public Message[] executeV2() {
        Message[] stats;
        Vector[] columns;
        CollectorTable[] tables;
        Object[] record = null;
        String new_line = null;
        String[] headers;
        String[] nddCmd;
        String driver;
        String configvar;
        BufferedReader pid_output = null;
        BufferedReader pid_error = null;
        Process pid = null;
        String NDDCOMMAND = null;
	
	/*
	 * Get the table information for this collector.
	 */
        tables = getTables();
	
        /*
         * Allocate an array of Vectors to contain the database
         * table column information for this collector. Allocate
         * a vector for each database table used by the collector.
	 */
        columns = new Vector[TABLENAME.length];
	
        /*
         * Allocate the array of Message objects to be returned from the
         * collector. Allocate a Message object for each database table.
         */
        stats = new Message[TABLENAME.length];
	
        /*
         *  Instantiate the message array.
         */
        for (int i = 0; i < TABLENAME.length; i++) {
            stats[i] = new Message(TABLENAME[i]);
	    
            columns[i] = tables[i].getColumns();
	    
            /*
             * Fill in the column headers for the current table.
             */
            headers = new String[columns[i].size()];
            for (int j = 0; j < columns[i].size(); j++) {
                headers[j] =
		    ((CollectorTable.Column) columns[i].elementAt(j)).getName();
            }
            stats[i].getDataVector().addElement(headers);
        }
	
        /*
         * Get the values specified on the collector instance for
         * the parameters.
         */
        Vector driverName = getParameterValues("DRIVER_NAME");
        Vector driverAttr = getParameterValues("DRIVER_ATTRIBUTE");
	
        /*
         * Remove the spaces or nulls in the parameters vectors.
         */
        RemoveNulls(driverName);
        RemoveNulls(driverAttr);
	
        /*
         * For each table, fill in the data records.
         */
        try {
            File ndd_cmd = null;
            String os_name = System.getProperty("os.name");
            if (os_name.equals("SunOS"))
                NDDCOMMAND = NDDCOMMAND_SUNOS;
            else
                NDDCOMMAND = NDDCOMMAND_HPUX;
	    
            ndd_cmd = new File(NDDCOMMAND);
	    
            if (!ndd_cmd.exists()) {
		/*
		 * Return an error message indicating that the command
		 * needed by the collector was not found. The message
		 * is logged to the client message log.
		 */
                return new Message[]
		    {
			errorMessage("The " + NDDCOMMAND +
				     " command was not found.")
		    };
            }
	    
            if ((driverName.isEmpty()) && (driverAttr.isEmpty()))
		/*
		 * Return an error message if both required parameters
		 * were not specified.
		 */
                return new Message[]
                {
		    errorMessage(
				 "The TCP/IP driver name and driver attribute must " +
				 "be specified as parameters.")
                };
	    
            if (driverName.isEmpty())
		/*
		 * Return an error message if the DRIVER_NAME parameter
		 * was not specified.
		 */
                return new Message[]
                {
		    errorMessage("The TCP/IP driver name must be specified.")
                };
	    
            if (driverAttr.isEmpty())
		/*
		 * Return an error message if the DRIVER_ATTRIBUTE parameter
		 * was not specified.
		 */
                return new Message[]
                {
		    errorMessage("The TCP/IP driver attribute must be specified.")
                };
	    
            if (driverName.size() != driverAttr.size())
		/*
		 * Return an error message if the number of parameters
		 * do not match.
		 */
                return new Message[]
                {
		    errorMessage(
				 "At least one driver name does not have a matching driver attribute.")
                };
	    
            for (int i = 0; i < driverName.size(); i++) {
                driver = driverName.elementAt(i).toString().trim();
                configvar = driverAttr.elementAt(i).toString().trim();
		
                nddCmd =
                    new String[]
                    {
                        "/bin/sh", "-c", "LANG=C " + NDDCOMMAND +
			"  -get " + driver + "  " + configvar
		    };
		
                try {
                    /*
                     * Execute the ndd command for the specified driver
                     * and attribute.
                     */
                    pid = Runtime.getRuntime().exec(nddCmd);
                }
                catch (IOException e) {
                    return new Message[]
			{
			    errorMessage("An error occurred running the command: "
                                         + NDDCOMMAND )
			};
                }
		
                /*
                 * Initialize output stream.
                 */
		
                pid_output =
		    new BufferedReader(
				       new InputStreamReader(pid.getInputStream()));
		
                /*
                 * Read the output stream as early as possible to avoid buffer
                 * overflow from large output.
                 */
		
                String value = "";
                while ((new_line = pid_output.readLine()) != null) {
                    value = new_line;
		    
                    /*
                     * Check if the driver attribute is valid for a specified
                     * driver.
                     */
                    if (value.length() > 20 &&
                        value.substring(0, 20).equals("name is non-existent"))
			/*
			 * Return an error message indicating collector failure.
			 */
                        return new Message[]
                        {
			    errorMessage("The " + configvar + " attribute " +
					 "is not valid for the specified driver, " +
					 driver + ".")
                        };
		    
                    record = new Object[columns[0].size()];
                    /*
                     * record[0] stores the driver name.
                     */
                    record[0] = driver;
                    /*
                     * record[1] stores the driver attribute.
                     */
                    record[1] = configvar;
		    
                    //
                    try {
                        record[2] = new Integer(value);
                    }
                    catch (NumberFormatException nfe) {
                        break;
                    }
		    
                    /*
                     * Add the record to the message vector.
                     */
                    stats[0].getDataVector().addElement(record);
                }
		
                /*
                 * Wait for the process to complete.
                 */
                int retVal = pid.waitFor();
		
                if (retVal != 0) {
                    pid_error = new BufferedReader(new InputStreamReader(pid.getErrorStream()));
                    String err_data;
                    StringBuffer errorLineBuf = new StringBuffer("");
                    while ((err_data = pid_error.readLine()) != null) {
                        errorLineBuf.append(" ").append(err_data);
                    }
		    
                    String errorLine = errorLineBuf.toString();
                    if (errorLine == null)
                        errorLine = "";
		    
                    /*
                     * check if the driver name is valid.
                     */
                    if (retVal == 1)
			/*
			 * Return an error message indicating that the TCP/IP
			 * driver name is not valid or does not exist.
			 */
                        return new Message[]
                        {
			    errorMessage("The value specified for the TCP/IP driver name, "
					 + driver + " is not valid or does not exist.")
                        };
		    /*
		     * Return an error message indicating collector failure.
		     * The message is logged in the client message log.
		     */
                    return new Message[]
			{
			    errorMessage("The " + NDDCOMMAND + " command " +
					 "returned the following error message: " +
					 errorLine + ".")
			};
                }
            }
        }
        catch (IOException io) {
	    /*
	     * Return an error message indicating that an unexpected error
	     * occurred while reading the output of the command.
	     */
            return new Message[] {
		errorMessage(
			     "An error occurred while reading the output from the "
			     + NDDCOMMAND + " command.")
            };
        }
	
        catch (Exception e) {
	    /*
	     * Return error message indicating collector failure.
	     * Message will be logged in the standard client error log.
	     */
            return new Message[] {
		errorMessage("An exception occurred. The exceptiom was: " +
			     e.getMessage())
            };
        }
        /*
         * Close the output and error streams.
         */
        finally {
            try {
                if (pid != null)
                    pid.destroy();
                if (pid_output != null)
                    pid_output.close();
                if (pid_error != null)
                    pid_error.close();
            }
            catch (IOException e) {
            }
        }
	
	/*
	 * Return the gathered collector data.
	 */
        return stats;
    }
    
    /*
     * The method RemoveNulls removes the spaces or nulls in the parameters vectors.
     */
    void RemoveNulls(Vector param) {
        for (int i = 0; i < param.size(); i++) {
            if (param.elementAt(i) == null || param.elementAt(i).equals("")) {
                param.remove(i);
                i--;
            }
        }
    }
    
}
